from collections import namedtuple

from sklearn.base import BaseEstimator
from sklearn.ensemble import RandomForestClassifier
from sklearn.linear_model import OrthogonalMatchingPursuit

from bolsonaro import LOG_PATH
from bolsonaro.error_handling.logger_factory import LoggerFactory
from bolsonaro.models.omp_forest import OmpForest, SingleOmpForest
import numpy as np

class OmpForestBinaryClassifier(SingleOmpForest):

    DEFAULT_SCORE_METRIC = 'indicator'

    def __init__(self, models_parameters):
        estimator = RandomForestClassifier(n_estimators=models_parameters.forest_size,
                                           random_state=models_parameters.seed, n_jobs=-1)
        super().__init__(models_parameters, estimator)

    def _check_classes(self, y):
        assert len(set(y).difference({-1, 1})) == 0, "Classes for binary classifier should be {-1, +1}"

    def fit(self, X_forest, y_forest, X_omp, y_omp):
        self._check_classes(y_forest)
        self._check_classes(y_omp)

        return super().fit(X_forest, y_forest, X_omp, y_omp)


    def score(self, X, y, metric=DEFAULT_SCORE_METRIC):
        """
        Evaluate OMPForestClassifer on (`X`, `y`) using `metric`

        :param X:
        :param y:
        :param metric: might be "indicator"
        :return:
        """
        predictions = self.predict(X)

        if metric == 'indicator':
            evaluation = np.abs(np.mean(np.abs(np.sign(predictions) - y) - 1))
        else:
            raise ValueError("Unsupported metric '{}'.".format(metric))

        return evaluation


class OmpForestMulticlassClassifier(BaseEstimator):
    def __init__(self, models_parameters):
        self._models_parameters = models_parameters
        self._base_forest_estimators = RandomForestClassifier(n_estimators=models_parameters.forest_size,
                                           random_state=models_parameters.seed, n_jobs=-1)
        self._logger = LoggerFactory.create(LOG_PATH, __name__)




if __name__ == "__main__":
    forest = RandomForestClassifier(n_estimators=10)
    X = np.random.rand(10, 5)
    y = np.random.choice([-1, +1], 10)
    forest.fit(X, y)
    print(forest.predict(np.random.rand(10, 5)))